!-------------------------------------------------------------------------------

!                      Code_Saturne version 4.0.3
!                      --------------------------
! This file is part of Code_Saturne, a general-purpose CFD tool.
!
! Copyright (C) 1998-2013 EDF S.A.
!
! This program is free software; you can redistribute it and/or modify it under
! the terms of the GNU General Public License as published by the Free Software
! Foundation; either version 2 of the License, or (at your option) any later
! version.
!
! This program is distributed in the hope that it will be useful, but WITHOUT
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
! FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
! details.
!
! You should have received a copy of the GNU General Public License along with
! this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
! Street, Fifth Floor, Boston, MA 02110-1301, USA.

!-------------------------------------------------------------------------------

!===============================================================================
! Purpose:
! -------

!> \file cs_user_extra_operations-nusselt_calculation.f90
!>
!> \brief This function is called at the end of each time step, and has a very
!>  general purpose
!>  (i.e. anything that does not have another dedicated user subroutine)
!>
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
! Arguments
!______________________________________________________________________________.
!  mode           name          role
!______________________________________________________________________________!
!> \param[in]     nvar          total number of variables
!> \param[in]     nscal         total number of scalars
!> \param[in]     dt            time step (per cell)
!_______________________________________________________________________________

subroutine cs_f_user_extra_operations &
 ( nvar   , nscal  ,                                              &
   dt     )

!===============================================================================

!===============================================================================
! Module files
!===============================================================================

use paramx
use dimens, only: ndimfb
use pointe
use numvar
use optcal
use cstphy
use cstnum
use entsor
use lagpar
use lagran
use lagdim
use parall
use period
use ppppar
use ppthch
use ppincl
use mesh
use field
use field_operator

!===============================================================================

implicit none

! Arguments

integer          nvar   , nscal

double precision dt(ncelet)
!< [loc_var_f_user]
! Local variables

integer iscal, ivar, iortho, iprev
integer inc, iccocg, ilelt, irangv
integer ifac, iel, npoint, iel1, irang1, ii, iun, impout, nlelt, neltg

double precision diipbx, diipby, diipbz
double precision xtbulk, xubulk, xyz(3), xtb, xub, tfac, lambda, xab


integer, allocatable, dimension(:) :: lstelt
double precision, dimension(:), pointer :: coefap, coefbp, cofafp
double precision, allocatable, dimension(:,:) :: grad
double precision, allocatable, dimension(:) :: treco,treglo,treloc
double precision, allocatable, dimension(:) :: xnusselt
double precision, allocatable, dimension(:) :: xabs, xabsg
double precision, dimension(:,:), pointer :: vel
double precision, dimension(:), pointer :: cvar, viscl

double precision Ux, Uy, xc, yc


integer ifac1,ifac2, iel2
double precision xmin,xmax,ymin,ymax,xmid_x,xmid_y,xLength,xNum,xrhom,xdelx,xdelz,xfac,xTm
double precision xUmx,xUmx_max,adim_xUmx,xUmx_max_y,xUmy,xUmy_max,adim_xUmy,xUmy_max_x,xUmxDim_max,xUmyDim_max
double precision xUmx_max_x, xUmy_max_y
double precision xTh,xTc,xNu_lw,xgradT





double precision :: height, prandtl, qwall
parameter (height = 1.0d0)
parameter (prandtl = 1.0d0)
parameter (qwall = 1.0d0)
!< [loc_var_f_user]
!***********************************************************************


! Calculation of the Nusselt number
!< [nusselt_number]
!if (ntcabs.eq.ntmabs) then
if (6.eq.6) then
  ! reached max iterations
  allocate(lstelt(max(ncel,nfac,nfabor)))
  call field_get_val_v(ivarfl(iu), vel)

  iscal = iscalt         ! temperature scalar number
  ivar =  isca(iscal)    ! temperature variable number
  call field_get_val_s(iprpfl(iviscl), viscl)
  call field_get_coefa_s(ivarfl(ivar), coefap)
  call field_get_coefb_s(ivarfl(ivar), coefbp)








  ! --> Compute the thickness of the domain (in the z direction)
  ifac1 = idebty(isymet)
  ifac2 = min(idebty(isymet)+int((ifinty(isymet)-idebty(isymet))/2)+1,ifinty(isymet))
  xdelz = abs(cdgfbo(3,itrifb(ifac1))-cdgfbo(3,itrifb(ifac2)))
  ! --> Compute the geometric characteristics of the computational domain and the
  !     temperatures of the hot and cold walls.
  !===============================================================================

  ifac1 = idebty(iparoi)
  ifac2 = ifinty(iparoi)

  xmin =  1.0d+12
  xmax = -1.0d+12
  xTc = 1.0d+12
  xTh = -1.0d+12
  ymin = 1.0d+12
  ymax = -1.0d+12

  do ifac = itrifb(ifac1), itrifb(ifac2) ! cycle through the wall boundary faces

    if (coefap(ifac).gt.0.0d0) then ! Choose the fixed temperature (vertical) walls

    ! Find the X positions of the vertical boundaries and the temperature extremas at those boundaries
      if (cdgfbo(1,ifac).lt.xmin) then
        xmin = cdgfbo(1,ifac)
        xTh = max(xTh,coefap(ifac))
      else if (cdgfbo(1,ifac).gt.xmax) then
        xmax = cdgfbo(1,ifac)
        xTc = min(xTc,coefap(ifac))
      end if    

    else ! Choose the adiabatic (horizontal) walls

    ! Find the Y positions of the horizontal boundaries
      if (cdgfbo(2,ifac).lt.ymin) then
        ymin = cdgfbo(2,ifac)
      else if (cdgfbo(2,ifac).gt.ymax) then
        ymax = cdgfbo(2,ifac)
      end if

    end if

  end do

  xLength = xmax-xmin ! Computational domain length
  xmid_x = (xmin+xmax)*0.5d0 ! Vertical mid-point
  xmid_y = (ymin+ymax)*0.5d0 ! Horizontal mid-point

  !--> Find the maximum velocity in the X direction at the central horizontal plane
  !    (the velocity is adimensionalised for comparison with the reference results)
  !================================================================================
  xNum = 0.0d0
  xUmx_max = 0
  xUmx_max_y = 0
  ro0 = 1.2039
  cp0 = 1004.84


  do ifac = 1, nfac

    iel1 = ifacel(1,ifac)
    iel2 = ifacel(2,ifac)
                
    if ((xyzcen(1,iel1)-xmid_x)*(xyzcen(1,iel2)-xmid_x) .lt. 0.0d0) then !find mid-face with straddling cells

      xUmx = max(vel(1,iel1), vel(1,iel2))


      !Maximum X velocity and location
      !-------------------------------
!      adim_xUmx = ro0*xUmx*xLength/visls0(1)
      adim_xUmx = ro0*cp0*xUmx*xLength/visls0(1)
      if (abs(adim_xUmx).gt.abs(xUmx_max)) then
        xUmxDim_max = xUmx
        xUmx_max = adim_xUmx
        xUmx_max_y = xyzcen(2,iel1)
      end if

    end if

  enddo

  !--> Find the maximum velocity in the Y direction at the central horizontal plane
  !    (the velocity is adimensionalised for comparison with the reference results)
  !================================================================================

  xUmy_max = 0
  xUmy_max_x = 0

  do ifac = 1, nfac
    iel1 = ifacel(1,ifac)
    iel2 = ifacel(2,ifac)

    if ((xyzcen(2,iel1)-xmid_y)*(xyzcen(2,iel2)-xmid_y) .lt. 0.0d0) then
      xUmy = max(vel(2,iel1), vel(2,iel2))
      !Maximum Y velocity and location
      !-------------------------------
      adim_xUmy = ro0*cp0*xUmy*xLength/visls0(1)
      if (abs(adim_xUmy).ge.abs(xUmy_max)) then
        xUmyDim_max = xUmy
        xUmy_max = adim_xUmy
        xUmy_max_x = xyzcen(1,iel1)
      end if
    end if
  enddo

  !write(*,*) 'max speed X ', xUmxDim_max, '[m/s] @ y=', xUmx_max_y, ' VxBar=', xUmx_max, '[] ---- max speed Y ', xUmyDim_max, '[m/s] @ x=', xUmy_max_x, ' VyBar=', xUmy_max, '[]'
  write(*,*) xUmxDim_max, xUmx_max_y, xUmx_max, xUmyDim_max, xUmy_max_x, xUmy_max

  xUmx_max = 0
  xUmy_max = 0
  do iel = 1, ncel
     Ux = vel(1,iel)*ro0*cp0*xLength/visls0(1)
     Uy = vel(2,iel)*ro0*cp0*xLength/visls0(1)
     xc = xyzcen(1,iel)
     yc = xyzcen(2,iel)
     if (abs(Ux).ge.abs(xUmx_max)) then
       xUmx_max = Ux
       xUmx_max_x = xc
       xUmx_max_y = yc
     end if
     if (abs(Uy).ge.abs(xUmy_max)) then
       xUmy_max = Uy
       xUmy_max_x = xc
       xUmy_max_y = yc
     end if
   enddo
  write(*,*) 'max Ux ', xUmx_max, xUmx_max_x, xUmx_max_y
  write(*,*) 'max Uy ', xUmy_max, xUmy_max_x, xUmy_max_y


endif

return
end subroutine cs_f_user_extra_operations
!< [bulk_nusselt]
subroutine sortc2(tab1, tab2, n)
!================
!< [loc_var_sortc2]
implicit none
integer n
double precision tab1(n), tab2(n)

integer ns, ii, jj, kk
double precision tabmin, t2
!< [loc_var_sortc2]
!< [body_sortc2]
ns = 1
do ii = 1, n-1
  tabmin = 1.d20
  kk = 0
  do jj = ns,n
    if (tabmin.gt.tab1(jj)) then
      tabmin = tab1(jj)
      kk = jj
    endif
  enddo
  t2 = tab2(kk)
  tab1(kk) = tab1(ns)
  tab2(kk) = tab2(ns)
  tab2(ns) = t2
  tab1(ns) = tabmin
  ns = ns + 1
enddo

return
end subroutine sortc2
!< [body_sortc2]
